package Merkato::Buyer;

require 5.005_62;
use strict;
use warnings;

use Data::Dumper;

require Exporter;
use XML::Simple;
use HTTP::Request::Common qw(POST);
use LWP::UserAgent;

use Carp;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Merkato::Buyer ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
new view start stop
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);

our $VERSION = '$Revision: 1.27 $';

use vars qw($AUTOLOAD);

my %moneys = (
	dollars => '$',
	cents	=> 'c',
    euros   => 'EUR'
);

my %valuations = (
    'Budget Valuation with Limits'  => 'BoundedBudgetValuation',
    'Square Root Valuation'         => 'RobertsValuation', 
    'Budget Valuation'              => 'BudgetValuation',
    'Linear Valuation'              => 'LinearValuation',
    'Parabolic Valuation'           => 'ParabolicValuation',
    'Logarithmic Valuation'         => 'LogarithmicValuation'
);
my %valuation_names = reverse %valuations;

# Preloaded methods go here.
sub new{
   	my $that = shift;
	my ($server, $port, $username, $password, $default_garage, $attr) = @_;
	
	croak('Usage: $class->new($server, $port ,$user ,$passwd [,\%attr])')
        	if (($attr and not ref $attr) or ref $password);

   	my %args = ( 
		TimeUnit	=> 'month',
 		MoneyUnit	=> 'dollars',
		RateUnit	=> 'Mbps',
       	Server		=> $server,
		Port		=> $port,
		Username	=> $username,
		Password	=> $password,
		Garage          => $default_garage,
        Debug		=> 0,
		ref $attr ? %$attr : ()
   	);
	$args{'UserAgent'} = LWP::UserAgent->new();
   	$args{'_reservations'} = ();
	my $class = ref($that)||$that;
   	my $self  = { %args };
   	bless($self, $class);
   	return $self;
} 

sub AUTOLOAD{
        my $self = shift;
        my $type = ref($self)||croak "$self is not an object";
        my $name = $AUTOLOAD;
        $name =~ s/.*://;
        unless (exists $self->{$name} ){
                croak "Can't access '$name' field in object of class $type";
        }
        if (@_){
                return $self->{$name} = shift;
        }
        else {
                return $self->{$name};
       }
}

sub DESTROY { }

sub _get_xml_hash{
	my $self = shift;
	my ($action, $garage) = @_;
	my $tree = {
		'action' => {
                	'name' => $action,
        	},
        	'Identity' => {
                	'name' => $self->{Username},
                	'password' => $self->{Password}
        	}
	}; 
	if (!defined($garage) && defined($self->{Garage})) {
	    $garage = $self->{Garage};
	}
	if (defined($garage)){
	    my $server = $self->{Server};
	    my $port   = $self->{Port};
	    $tree->{action}->{garageURL} = "http://$server:$port/merkato/$garage";
	} 
	return $tree;
}

sub _send_xml{
	my $self = shift;
	my ($tree) = @_;

	my $server = $self->{Server};
	my $port   = $self->{Port};
	my $request = XMLout($tree, rootname=>'methodCall' );

	print "$request\n" if ($self->{'Debug'} > 1);

	$request =~ s/^\s+//mg;
	$request =~ s/\n//mg;

	my $req = POST "http://$server:$port/merkato/access",
               [ call => $request ];
	
	my $content = $self->{UserAgent}->request($req)->as_string;
	
	print $content if ($self->{'Debug'} > 1);
	$content =~ s/.+\n\n.+?\n//s;

	die "No output from server\n" if($content =~ /^\s*$/s);

	my $output = XMLin($content, keeproot => 1);
	
	if (exists($output->{'Exception'})){
		print Dumper $output if ($self->{'Debug'} > 0);
		die $output->{'Exception'}->{'message'} , "\n";
	} else {
		return $output;
	}
}

sub _in_array{
	my $self = shift;
	my ($value, @array) = @_;
	my $elem;
	foreach $elem (@array){
		return 1 if ($value eq $elem);
	}
	return 0;
}

sub available_time_units{
	return qw(min h day month);
}

sub available_money_units{
	return keys %moneys;
}

sub available_quantity_units{
	return qw(Mbps Kbps Gbps);
}

sub available_response_names{
    return qw(quick regular);
}

sub view {
	my $self = shift;
	my $tree = $self->_get_xml_hash('x-view');
	my $result = undef;
	eval { $result = $self->_send_xml($tree);};
	if ($@) {
        chomp $@;
		croak ( "viewing problem: $@");
	}
	return $result->{Buyer};
}

sub start{
	my $self = shift;
    my $tree = $self->_get_xml_hash('x-apply');
	$tree->{startRequested} = {};
	my $result = undef;
    eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
        croak ( "Problem Starting Agent: $@");
    }
    return 1;
}

sub stop{
    my $self = shift;
    my $tree = $self->_get_xml_hash('x-apply');
    $tree->{stopRequested} = {};
    my $result = undef;
    eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
        croak ( "Problem Stoping Agent: $@");
    }
    return 1;
}

sub auto_strategy{
	my $self = shift;
    my $tree = $self->_get_xml_hash('x-apply');
    $tree->{currentStrategySelected} = { 
	    'param' => {
		    'name' => 'string', 
		    'value' => 'Auto Strategy'
		},
		'-test' => ''
    };
    my $result = undef;
    eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
        croak ( "Problem Setting Strategy: $@");
    }
    return 1;
}

sub manual_strategy{
    my $self = shift;
    my $tree = $self->_get_xml_hash('x-apply');
    $tree->{currentStrategySelected} = {
        'param' => {
	        'name' => 'string',
           	'value' => 'Manual Strategy'
        },
		'-test' => ''
	};
    my $result = undef;
    eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
        croak ( "Problem Setting Strategy: $@");
    }
    return 1;
}

sub _set_valuation{
    my $self = shift;
    my ($valuation) = @_;
    
    my $tree = $self->_get_xml_hash('x-apply');
        $tree->{'currentValuationSelected'} = {
        'param' => {
            'name' => 'string',
            'value' => $valuation
         },
         '-test'=> ''
    };
    my $result = undef;
    eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
        croak ( "Problem Changing Valuation: $@");
    }
    return 1;
}

sub budget_valuation{
	my $self = shift;
    my ($budget) = @_;
	
    $self->_set_valuation('Budget Valuation') or 
        croak "Could not set Valuation\n"; 
    
    my $unit = $moneys{$self->{'MoneyUnit'}} . '/' . $self->{'TimeUnit'};
    my $tree = $self->_get_xml_hash('x-apply');
    $tree->{'valuationChanged'} = {
		'param' => {
            'name' => 'string',
       	    'value' => 'Budget Valuation'
        },
		'BudgetValuation' => {
			Value => {
				name => 'budget',
				displayValue => $budget,
				displayUnit  => $unit
			},
			'-test'=> ''
		},
		'-test'=> ''
	};
	my $result = undef;
	eval { $result = $self->_send_xml($tree);};
	if ($@) {
        chomp $@;
        croak ( "Problem Setting Valuation: $@");
    }
	return 1;
}
	
sub budget_valuation_with_limits{
    my $self = shift;
    my ($budget, $qmin, $qmax) = @_;
    $self->_set_valuation('Budget Valuation with Limits') or 
        croak "Could not set Valuation\n"; 
    
	my $value_unit = $moneys{$self->{MoneyUnit}} . '/' . $self->{TimeUnit};
        my $rate_unit = $self->{RateUnit};
	
	my $tree = $self->_get_xml_hash('x-apply');
    $tree->{valuationChanged} = {
        'param' => {
            'name'  => 'string',
            'value' => 'Budget Valuation with Limits'
        },
        'BoundedBudgetValuation' => {
            Value => {
                name         => 'budget',
                displayValue => $budget,
                displayUnit  => $value_unit
            },
		    Quantity => [
		        {
		            name         => 'qmin',
		            displayValue => $qmin,
		            displayUnit  => $rate_unit
		        },
		        {
                    name         => 'qmax',
                    displayValue => $qmax,
                    displayUnit  => $rate_unit
                }
	        ]       
        },
        '-test'=> ''
    };
    my $result = undef;
    eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
        croak ( "Problem Setting Valuation: $@");
    }
    return 1;
}

sub square_root_valuation{
    my $self = shift;
    my ($vmax, $qmax) = @_;
    
    $self->_set_valuation('Square Root Valuation') or
        croak "Could not set Valuation\n";
    
	my $value_unit = $moneys{$self->{MoneyUnit}} . '/' . $self->{TimeUnit};
    my $rate_unit = $self->{RateUnit};
	
	my $tree = $self->_get_xml_hash('x-apply');
    $tree->{valuationChanged} = {
	    'param' => {
      	    'name'  => 'string',
            'value' => 'Square Root Valuation'
		},
        'RobertsValuation' => {
            Value => {
                name         => 'vmax',
                displayValue => $vmax,
                displayUnit  => $value_unit
            },
	        Quantity => {
                name         => 'qmax',
                displayValue => $qmax,
                displayUnit  => $rate_unit
	        },
	        '-test'=> ''
        },
        '-test'=> ''
    };

    my $result = undef;
    eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
        croak ( "Problem Setting Valuation: $@");
    }
    return 1;
} 

sub linear_valuation{
    my $self = shift;
    my ($vmax, $qmax) = @_;
    

    $self->_set_valuation('Linear Valuation') or
        croak "Could not set Valuation\n";
     
    my $value_unit = $moneys{$self->{MoneyUnit}} . '/' . $self->{TimeUnit};
    my $rate_unit = $self->{RateUnit};

    my $tree = $self->_get_xml_hash('x-apply');
    $tree->{valuationChanged} = {
        'param' => {
            'name'  => 'string',
            'value' => 'Linear Valuation'
        },
        'LinearValuation' => {
            Value => {
                name         => 'vmax',
                displayValue => $vmax,
                displayUnit  => $value_unit
            },
            Quantity => {
                name         => 'qmax',
                displayValue => $qmax,
                displayUnit  => $rate_unit
            },
            '-test'=> ''
        },
        '-test'=> ''
    };

    my $result = undef;
    eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
        croak ( "Problem Setting Valuation: $@");
    }
    return 1;
}

sub logarithmic_valuation{
    my $self = shift;
    my ($vmax, $qmax) = @_;
    
    $self->_set_valuation('Logarithmic Valuation') or
        croak "Could not set Valuation\n";

 
    my $value_unit = $moneys{$self->{MoneyUnit}} . '/' . $self->{TimeUnit};
    my $rate_unit = $self->{RateUnit};

    my $tree = $self->_get_xml_hash('x-apply');
    $tree->{valuationChanged} = {
        'param' => {
       		'name' => 'string',
       		'value' => 'Logarithmic Valuation'
        },
        'LogarithmicValuation' => {
        	Value => {
           		name         => 'vmax',
           	    displayValue => $vmax,
                displayUnit  => $value_unit
            },
            Quantity => {
                name         => 'qmax',
                displayValue => $qmax,
                displayUnit  => $rate_unit
             },
             '-test'=> ''
        },
       	'-test'=> ''
    };

    my $result = undef;
    eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
        croak ( "Problem Setting Valuation: $@");
    }
    return 1;
}

sub parabolic_valuation{
    my $self = shift;
    my ($vmax, $qmax) = @_;
   
    $self->_set_valuation('Parabolic Valuation') or
        croak "Could not set Valuation\n";

     
    my $value_unit = $moneys{$self->{MoneyUnit}} . '/' . $self->{TimeUnit};
    my $rate_unit = $self->{RateUnit};

    my $tree = $self->_get_xml_hash('x-apply');
    $tree->{valuationChanged} = {
        'param' => {
            'name' => 'string',
           	'value' => 'Parabolic Valuation'
        },
        'ParabolicValuation' => {
        	Value => {
           	    name         => 'vmax',
                displayValue => $vmax,
                displayUnit  => $value_unit
            },
            Quantity => {
        	    name         => 'qmax',
                displayValue => $qmax,
                displayUnit  => $rate_unit
            },
            '-test'=> ''
        },
        '-test'=> ''
    };

    my $result = undef;
    eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
        croak ( "Problem Setting Valuation: $@");
    }
    return 1;
}

sub _get_valuation {
    my $self = shift;
    my $view = $self->view();
    
    return $valuations{$view->{'Selections'}->{'valuation'}};
}

sub traffic_response {
    my $self = shift;
    my %args = (
        Response        => undef,
        Margin          => undef,
        TimeFrame       => undef,
        Threshold       => undef,
        EstimatedPeak   => undef,
        @_,
    );

    my $response       = $args{'Response'};
    my $margin         = $args{'Margin'};
    my $time_frame     = $args{'TimeFrame'};
    my $active         = $args{'Active'};
    my $threshold      = $args{'Threshold'};
    my $estimated_peak = $args{'EstimatedPeak'};
    
    unless(defined($response)){
        $self->_response_off();
        return 1;
    }

    if ($response =~ /regular/i){
        unless(defined($margin)){
            croak "You must specify a Margin for Regular Response";
        }
        unless(defined($time_frame)){
            croak "You must specify a TimeFrame for Regular Response";
            unless ($time_frame > 0) {
                croak "TimeFrame must be positive";                     
            }
    }
        $self->_regular_response($margin, $time_frame);
   
     } elsif ($response =~ /quick/i){
        unless(defined($margin)){
            croak "You must specify a Margin for Quick Response";
        }
        unless(defined($time_frame)){
            croak "You must specify a TimeFrame for Quick Response";
            unless ($time_frame > 0) {
                croak "TimeFrame must be positive";
            } 
        }
        unless(defined($threshold)){
            croak "You must specify a Threshold for Quick Response";
        }
        unless(defined($estimated_peak)){
            croak "You must specify a EstimatedPeak for Quick Response";
        }
        $self->_quick_response($margin, $time_frame, $threshold, $estimated_peak);
    
    } else{
        croak 'Invalid Response type';
    }
    return 1   
}

sub _response_off{
    my $self = shift;
    my $view = $self->view();
    my $valuation = $valuations{$view->{'Selections'}->{'valuation'}};
    my $val_name  = $valuation_names{$valuation};
   
    my $response_type; 
    if(exists($view->{$valuation}->{'QuickResponseTrafficValuationModulator'})){
        $response_type = 'QuickResponseTrafficValuationModulator';
    } else {
        $response_type = 'TrafficValuationModulator';
    }
    
    my $response_hash = $view->{$valuation}->{$response_type};
    $response_hash->{'active'} = 'false';
    $response_hash->{'-test'}  = '';
    
    my $tree = $self->_get_xml_hash('x-apply');
    $tree->{'valuationChanged'} = {
        'param' => {
            'name'  => 'string',
            'value' => $val_name
        },
        '-test'=> ''
    };
    
    $tree->{'valuationChanged'}->{$valuation}->{$response_type} = $response_hash;
    $tree->{'valuationChanged'}->{$valuation}->{'-test'}  = '';

    my $result = undef;
    eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
        croak ( "Problem Unsetting Valuation: $@");
    }
    return 1;
}

sub _regular_response {
    my $self = shift;
       
    my ($margin,$measured_time_frame) = @_;

    my $valuation = $self->_get_valuation();
    my $val_name  = $valuation_names{$valuation};
    my $time_unit = $self->{'TimeUnit'};
    my $rate_unit = $self->{'RateUnit'};

    my $tree = $self->_get_xml_hash('x-apply');
    $tree->{'valuationChanged'} = {
	    'param' => {
         	'name'  => 'string',
            'value' => $val_name
        },
        $valuation => {
		    'TrafficValuationModulator' => {
			    'dataCollector' => 'TrafficDataCollector',
			    'active'        => 'true',
			    'Duration'      => {
				    'name'          => 'measuredTimeFrame',
				    'displayValue'  => $measured_time_frame,
				    'displayUnit'   => $time_unit
			    },
			    'Quantity'      => {
				    'name'         => 'margin',
				    'displayValue' => $margin,
				    'displayUnit'  => $rate_unit
			    }
		    },
            '-test'=> ''
       	},
       	'-test'=> ''
    };

    my $result = undef;
    eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
        croak ( "Problem Setting Valuation: $@");
    }
    return 1;
}

sub _quick_response {

    my $self = shift;
    my ($margin,$measured_time_frame,$threshold,$estimated_traffic_peak) = @_;

    my $tree = $self->_get_xml_hash('x-apply');

    my $valuation = $self->_get_valuation();
    my $val_name  = $valuation_names{$valuation};
    my $time_unit = $self->{TimeUnit};
    my $rate_unit = $self->{RateUnit};

    $tree->{'valuationChanged'} = {
        'param' => {
            'name'  => 'string',
            'value' => $val_name
        },

        $valuation => {
            'QuickResponseTrafficValuationModulator' => {
                'dataCollector' => 'TrafficDataCollector',
                'active'        => 'true',
                'quickResponseActive'      => 'true',
                'Duration'      => {
                    'name'          => 'measuredTimeFrame',
                     'displayValue' => $measured_time_frame,
                     'displayUnit'  => $time_unit
                },
			    Quantity        => [
                    {
                        name         => 'margin',
                        displayValue => $margin,
                        displayUnit  => $rate_unit
                    },
                    {
                        name         => 'threshold',
                        displayValue => $threshold,
                        displayUnit  => $rate_unit
                    },
				    {
				        name         => 'estimatedTrafficPeak',
				        displayValue => $estimated_traffic_peak,	
				        displayUnit  => $rate_unit
				    }
                ]
            },
            '-test'=> ''
        },
        '-test'=> ''
    };

    my $result = undef;
    eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
        croak ( "Problem Setting Valuation: $@");
    }
    return 1;
}


## reservations

sub submit_reservation{
 	my $self = shift;
	my %args = ( 
       	Quantity	=> undef, 
       	Start		=> undef, 
       	End	        => undef,
	    Duration	=> undef,
       	@_,         # argument pair list goes here
    	);
	
	unless (defined($args{'Quantity'})){
		croak 'Quantity must be specified' 
	}		

	unless (defined($args{'End'}) or defined($args{'Duration'})){
		croak 'End or Duration must be specified';
	}

	my $quantity  = $args{'Quantity'};
    my $rate_unit  = $self->{'RateUnit'};
	my $time_unit  = $self->{'TimeUnit'};
    my $money_unit = $moneys{$self->{'MoneyUnit'}};
    
    my $tree = $self->_get_xml_hash('x-apply', 'reservation-garage');
	$tree->{'bidSubmitted'} = {
		'ReservationBid' => {
			'ReservedQuantity' => {
					name         => 'quantity',
                    			displayValue => $quantity,
                    			displayUnit  => $rate_unit
			},
			'-test'=> ''
		},
		'-test'=> ''
	};

	if (defined($args{'Duration'})){

		my $duration = $args{'Duration'};
		
		$tree->{'bidSubmitted'}->{'ReservationBid'}->{'ReservedQuantity'}->{'Duration'} = {
			name         => 'duration',
           	displayValue => $duration,
           	displayUnit  => $time_unit
		};

	}

	if (defined($args{'Start'}) && !defined($args{'End'})){

		my $start = $args{'Start'} * 1000;

		$tree->{'bidSubmitted'}->{'ReservationBid'}->{'ReservedQuantity'}->{'Time'} = {
			'name' => 'start',
            		'time' => $start
		};

	}

	if (defined($args{'End'}) && !defined($args{'Start'})){

        	my $end = $args{'End'} * 1000;

        	$tree->{'bidSubmitted'}->{'ReservationBid'}->{'ReservedQuantity'}->{'Time'} = {
            		'name' => 'end',
            		'time' => $end
        	};
    	}

	if (defined($args{'End'}) && defined($args{'Start'})){

		my $start = $args{'Start'} * 1000;
		my $end   = $args{'End'}   * 1000;

		$tree->{'bidSubmitted'}->{'ReservationBid'}->{'ReservedQuantity'}->{'Time'} = [
                        { 'name' => 'start',
                          'time' => $start
                        },
			{ 'name' => 'end',
                          'time' => $end
			}
                ];
	}


	my $result = undef;
	eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
      	croak ( "Problem Getting Reservation: $@");
    }

	$result = $result->{'ReservationBid'};
        
    my $id          = $result->{'sessionId'};
    my $cost        = $result->{'cost'};
    
    my $confirmtree = $self->_get_xml_hash('x-apply', 'reservation-garage');
    $confirmtree->{'bidConfirmed'} = {
       	'ReservationBid' => {
           	'sessionId' => $id,
        	'-test'=> ''
        },
        '-test'=> ''
    };
    #$confirmtree->{'bidConfirmed'}->{'ReservationBid'}->{'ReservedQuantity'} = 
    #   $tree->{'bidSubmitted'}->{'ReservationBid'}->{'ReservedQuantity'};
    #$confirmtree->{'bidConfirmed'}->{'ReservationBid'}->{'Cost'} = $result->{'Cost'};
     
    $confirmtree->{'bidConfirmed'}->{'ReservationBid'} = $result; 
     
    $self->{'_reservations'}->{$id}=$confirmtree;
    
    return ($id, $cost);
}

sub confirm_reservation {
    
    my $self = shift;
    my ($id) = @_;   

    my $tree = $self->{'_reservations'}->{$id};

    my $result = undef;
    eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
        croak ( "Problem Confirming Reservation: $@");
    }

    return 1;
}

sub get_current_reservations {
    my $self = shift;

    my $tree = $self->_get_xml_hash('x-view', 'reservation-garage');

    my $result = undef;
    eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
        croak ( "Problem viewing reservations: $@");
    }
    my $allocation = $result->{'Buyer'}->{'Allocation'};
    return $allocation;
}
 
1;
__END__
# Documentation follows

=head1 NAME

Merkato::Buyer - Perl extension for controling a Merkato Buyer Agent

=head1 SYNOPSIS

  use Merkato::Buyer;
  
  @time_names      = Merkato::Buyer->available_time_units;
  @money_names     = Merkato::Buyer->available_money_units;
  @bandwidth_names = Merkato::Buyer->available_quantity_units;
  @response_names  = Merkato::Buyer->available_response_names;  
  @valuation_names = Merkato::Buyer->available_valuation_names;

  $buyer = Merkato::Buyer->new(
	$server, $port, $username, $password, \%attr);
  
  $buyer->{RateUnit}  = 'Mbps';
  $buyer->{MoneyUnit} = 'dollars';
  $buyer->{TimeUnit}  = 'month' 

  $buyer_hashref = $buyer->view();
 
  $rv = $buyer->start();
  $rv = $buyer->stop();
  
  $rv = $buyer->manual_strategy;
  $rv = $buyer->auto_strategy;

  $rv = $buyer->budget_valuation($budget);
  $rv = $buyer->budget_valuation_with_limits($budget, $min_quantity, $max_quantity);
  $rv = $buyer->square_root_valuation($budget, $quantity);
  $rv = $buyer->linear_valuation($budget, $quantity);
  $rv = $buyer->logarithmic_valuation($budget, $quantity);
  $rv = $buyer->parabolic_valuation($budget, $quantity);
  
  $rv = $buyer->traffic_response( Response  => 'regular',
                            Margin    => $margin,
                            TimeFrame => $timeframe);

  $rv = $buyer->traffic_response( Response      => 'quick',
                            Margin        => $margin,
                            TimeFrame     => $timeframe,
                            Threshold     => $threshold,
                            EstimatedPeak => $peak);

  $rv = $buyer->traffic_response;
  
  $reservations = $buyer->get_current_reservations;
  
  ($id, $value) = $buyer->submit_reservation(
                        Quantity => $quantity,
                        Start    => $start,
                        End      => $end);
  
  ($id, $value) = $buyer->submit_reservation(
                        Quantity => $quantity,
                        Start    => $start,
                        Duration => $duration);
  ($id, $value) = $buyer->submit_reservation(
                        Quantity => $quantity,
                        Duration => $duration);
  
  ($id, $value) = $buyer->submit_reservation(
                        Quantity => $quantity,
                        End      => $end);

  $rv = $buyer->confirm_reservation($id);
 

=head1 DESCRIPTION

"Merkato::Buyer" provides an object interface to controling a Buyer agent on a merkato server.


=head2 Notation and Conventions

The following conventions are used in this document:
  $buyer    Buyer control object
  $rv       Return Value  (boolean: true=ok, false=error

=head1 CONSTRUCTOR

  $buyer = Merkato::Buyer->new(
    $server, $port, $username, $password, \%attr);

    Creates a "Merkato::Buyer" which is a reference to an object.  "new" takes as arguements, the server's address, the server's port, the username and the password of the buyer agent that you wish to control. 

=head1 METHODS

=item C<view>

  $buyer_hashref = $buyer->view();

Returns a reference to a hash which contains the parsed version of the xml which describes the buyer agent.

=item C<start>

  $rv = $buyer->start();

Sets the buyer to begin bidding for bandwidth

=item C<stop>

  $rv = $buyer->stop();

Stops the buyer from bidding for bandwidth

=item C<manual_strategy>

  $rv = $buyer->manual_strategy;

Sets the strategy of the buyer to be manual (effectivly disabeling bidding as there is no support for manual bidding in the XML API

=item C<auto_strategy>

  $rv = $buyer->auto_strategy;

Sets the strategy fo the buyer to be be the automatic strategy, to bid automatically

=item C<budget_valuation>

  $rv = $buyer->budget_valuation($budget);

Sets Budget Valuation to be the current valuation, using the given budget.

=item C<budget_valuation_with_limits>

  $rv = $buyer->budget_valuation_with_limits($budget, $min_quantity, $max_quantity);

Sets Budget Valuation with Limits to be the current valuation, using the given settings

=item C<square_root_valuation>

  $rv = $buyer->square_root_valuation($budget, $quantity);

Sets Square Root Valuation to be the current valuation, using the given settings.

=item C<linear_valuation>

  $rv = $buyer->linear_valuation($budget, $quantity);

Sets Linear Valuation to be the current valuation, using the given settings.

=item C<logarithmic_valuation>

  $rv = $buyer->logarithmic_valuation($budget, $quantity);

Sets Logarithmic Valuation to be the current valuation, using the given settings.

=item C<parabolic_valuation>

  $rv = $buyer->parabolic_valuation($budget, $quantity);

Sets Parabolic Valuation to be the current valuation, using the given settings.


=item C<traffic_response>

  $rv = $buyer->traffic_response( Response  => 'regular',
                            Margin    => $margin,
                            TimeFrame => $timeframe);
  or
  $rv = $buyer->traffic_response( Response      => 'quick',
                            Margin        => $margin,
                            TimeFrame     => $timeframe,
                            Threshold     => $threshold,
                            EstimatedPeak => $peak);
  or
  $rv = $buyer->traffic_response;

Turns on or off Traffic Responce. Called with no arguments, it turns off
Traffic Response.  (Note that you cannot change the type of traffic feedback by
using the APIs but only parameters for the current traffic feedback feature
given by the administrator. )

=item C<get_current_reservations>

  $reservations = $buyer->get_current_reservations;

Returns an array of references to hashes, containing all of the current reservations.

=item C<submit_reservation>

  ($id, $value) = $buyer->submit_reservation(
                        Quantity => $quantity,
                        Start    => $start,
                        End      => $end);
  or
  ($id, $value) = $buyer->submit_reservation(
                        Quantity => $quantity,
                        Start    => $start,
                        Duration => $duration);
  or
  ($id, $value) = $buyer->submit_reservation(
                        Quantity => $quantity,
                        Duration => $duration)

Submits a reservation,  If Start is omitted, now is assumed.

=item C<confirm_reservation>

  $rv = $buyer->confirm_reservation($id);
Confirms and starts a reservation.



=head1 AUTHOR

David Kapell, dkapell@invisiblehand.net

=head1 SEE ALSO

perl(1).

=cut
