package Merkato::Seller;

require 5.005_62;
use strict;
use warnings;

use Data::Dumper;

require Exporter;
use XML::Simple;
use HTTP::Request::Common qw(POST);
use LWP::UserAgent;

use Carp;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Merkato::Seller ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
new view start stop
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);

our $VERSION = '$Revision: 1.4 $';

use vars qw($AUTOLOAD);

my %moneys = (
	dollars => '$',
	cents	=> 'c',
    euros   => 'EUR'
);

my %valuations = (
    'Seller Valuation'              => 'LinearValuation',
);
my %valuation_names = reverse %valuations;

# Preloaded methods go here.
sub new{
   	my $that = shift;
	my ($server, $port, $username, $password, $default_garage, $attr) = @_;
	
	croak('Usage: $class->new($server, $port ,$user ,$passwd [,\%attr])')
        	if (($attr and not ref $attr) or ref $password);

   	my %args = ( 
		TimeUnit	=> 'month',
 		MoneyUnit	=> 'dollars',
		RateUnit	=> 'Mbps',
       	Server		=> $server,
		Port		=> $port,
		Username	=> $username,
		Password	=> $password,
                Garage          => $default_garage,
        Debug		=> 0,
		ref $attr ? %$attr : ()
   	);
	$args{'UserAgent'} = LWP::UserAgent->new();
   	$args{'_reservations'} = ();
	my $class = ref($that)||$that;
   	my $self  = { %args };
   	bless($self, $class);
   	return $self;
} 

sub AUTOLOAD{
        my $self = shift;
        my $type = ref($self)||croak "$self is not an object";
        my $name = $AUTOLOAD;
        $name =~ s/.*://;
        unless (exists $self->{$name} ){
                croak "Can't access '$name' field in object of class $type";
        }
        if (@_){
                return $self->{$name} = shift;
        }
        else {
                return $self->{$name};
       }
}

sub DESTROY { }

sub _get_xml_hash{
	my $self = shift;
	my ($action, $garage) = @_;
	my $tree = {
		'action' => {
                	'name' => $action,
        	},
        	'Identity' => {
                	'name' => $self->{Username},
                	'password' => $self->{Password}
        	}
	}; 

	if(!defined($garage) && defined($self->{Garage})) {
	    $garage = $self->{Garage};
	}

	if (defined($garage)){
	    my $server = $self->{Server};
	    my $port   = $self->{Port};
	    $tree->{action}->{garageURL} = "http://$server:$port/merkato/$garage/";
	}
	return $tree;
}

sub _send_xml{
	my $self = shift;
	my ($tree) = @_;

	my $server = $self->{Server};
	my $port   = $self->{Port};
	my $request = XMLout($tree, rootname=>'methodCall' );

	print "$request\n" if ($self->{'Debug'} > 0);

	$request =~ s/^\s+//mg;
	$request =~ s/\n//mg;

	my $req = POST "http://$server:$port/merkato/access",
               [ call => $request ];
	
	my $content = $self->{UserAgent}->request($req)->as_string;
	
	print $content if ($self->{'Debug'} > 1);
	
	$content =~ s/.+\n\n.+?\n//s;
    die "No output from server\n" if($content =~ /^\s*$/s);
		
	my $output = XMLin($content, keeproot => 1);
	
	if (exists($output->{'Exception'})){
		print Dumper $output if ($self->{'Debug'} > 1);
		die $output->{'Exception'}->{'message'} , "\n";
	} else {
		return $output;
	}
}

sub _in_array{
	my $self = shift;
	my ($value, @array) = @_;
	my $elem;
	foreach $elem (@array){
		return 1 if ($value eq $elem);
	}
	return 0;
}

sub available_time_units{
	return qw(min h day month);
}

sub available_money_units{
	return keys %moneys;
}

sub available_quantity_units{
	return qw(Mbps Kbps Gbps);
}

sub available_response_names{
    return qw(quick regular);
}

sub view {
	my $self = shift;
	my $tree = $self->_get_xml_hash('x-view');
	my $result = undef;
	eval { $result = $self->_send_xml($tree);};
	if ($@) {
        chomp $@;
		croak ( "viewing problem: $@");
	}
	return $result->{Seller};
}

sub start{
	my $self = shift;
    my $tree = $self->_get_xml_hash('x-apply');
	$tree->{startRequested} = {};
	my $result = undef;
    eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
        croak ( "Problem Starting Agent: $@");
    }
    return 1;
}

sub stop{
    my $self = shift;
    my $tree = $self->_get_xml_hash('x-apply');
    $tree->{stopRequested} = {};
    my $result = undef;
    eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
        croak ( "Problem Stoping Agent: $@");
    }
    return 1;
}

sub static_strategy{
	my $self = shift;
    my $tree = $self->_get_xml_hash('x-apply');
    $tree->{currentStrategySelected} = { 
	    'param' => {
		    'name' => 'string', 
		    'value' => 'Static Seller Strategy'
		},
		'-test' => ''
    };
    my $result = undef;
    eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
        croak ( "Problem Setting Strategy: $@");
    }
    return 1;
}

sub dynamic_strategy{
    my $self = shift;
    my $tree = $self->_get_xml_hash('x-apply');
    $tree->{currentStrategySelected} = {
        'param' => {
	        'name' => 'string',
           	'value' => 'Dynamic Seller Strategy'
        },
		'-test' => ''
	};
    my $result = undef;
    eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
        croak ( "Problem Setting Strategy: $@");
    }
    return 1;
}

sub _set_valuation{
    my $self = shift;
    my ($valuation) = @_;
    
    my $tree = $self->_get_xml_hash('x-apply');
        $tree->{'currentValuationSelected'} = {
        'param' => {
            'name' => 'string',
            'value' => $valuation
         },
         '-test'=> ''
    };
    my $result = undef;
    eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
        croak ( "Problem Changing Valuation: $@");
    }
    return 1;
}

sub seller_valuation{
    my $self = shift;
    my ($vmax, $qmax) = @_;
   
    $self->_set_valuation('Seller Valuation') or
        croak "Could not set Valuation\n";

     
    my $value_unit = $moneys{$self->{MoneyUnit}} . '/' . $self->{TimeUnit};
    my $rate_unit = $self->{RateUnit};

    my $tree = $self->_get_xml_hash('x-apply');
    $tree->{'valuationChanged'} = {
        'param' => {
            'name' => 'string',
           	'value' => 'Seller Valuation'
        },
        'LinearValuation' => {
        	Value => {
           	    name         => 'vmax',
                displayValue => $vmax,
                displayUnit  => $value_unit
            },
            Quantity => {
        	    name         => 'qmax',
                displayValue => $qmax,
                displayUnit  => $rate_unit
            },
            '-test'=> ''
        },
        '-test'=> ''
    };

    my $result = undef;
    eval { $result = $self->_send_xml($tree);};
    if ($@) {
        chomp $@;
        croak ( "Problem Setting Valuation: $@");
    }
    return 1;
}

sub _get_valuation {
    my $self = shift;
    my $view = $self->view();
    
    return $valuations{$view->{'Selections'}->{'valuation'}};
}

1;
__END__
# Documentation follows

=head1 NAME

Merkato::Seller - Perl extension for controling a Merkato Seller Agent

=head1 SYNOPSIS

  use Merkato::Seller;
  
  @time_names      = Merkato::Seller->available_time_units;
  @money_names     = Merkato::Seller->available_money_units;
  @bandwidth_names = Merkato::Seller->available_quantity_units;
  @response_names  = Merkato::Seller->available_response_names;  
  @valuation_names = Merkato::Seller->available_valuation_names;

  $seller = Merkato::Seller->new(
	$server, $port, $username, $password, \%attr);
  
  $seller->{RateUnit}  = 'Mbps';
  $seller->{MoneyUnit} = 'dollars';
  $seller->{TimeUnit}  = 'month' 

  $seller_hashref = $seller->view();
 
  $rv = $seller->start();
  $rv = $seller->stop();
  
  $rv = $seller->static_strategy();
  $rv = $seller->dynamic_strategy();

  $rv = $seller->seller_valuation($value, $quantity);
  
=head1 DESCRIPTION

"Merkato::Seller" provides an object interface to controling a Seller agent on a merkato server.


=head2 Notation and Conventions

The following conventions are used in this document:
  $seller    Seller control object
  $rv       Return Value  (boolean: true=ok, false=error

=head1 CONSTRUCTOR

  $seller = Merkato::Seller->new(
    $server, $port, $username, $password, \%attr);

    Creates a "Merkato::Seller" which is a reference to an object.  "new" takes as arguements, the server's address, the server's port, the username and the password of the seller agent that you wish to control. 

=head1 METHODS

=item C<view>

  $seller_hashref = $seller->view();

Returns a reference to a hash which contains the parsed version of the xml which describes the seller agent.

=item C<start>

  $rv = $seller->start();

Sets the seller to begin selling bandwidth

=item C<stop>

  $rv = $seller->stop();

Stops the seller from selling bandwidth

=item C<static_strategy>

  $rv = $seller->static_strategy;

Sets the strategy of the seller to be static

=item C<dynamic_strategy>

  $rv = $seller->dynamic_strategy;

Sets the strategy fo the seller to be be the dynamic strategy, to buy back extra bandwidth

=item C<seller_valuation>

  $rv = $seller->seller_valuation($value, $quantity);

Sets Seller Valuation to be the current valuation, using the given settings.

=head1 AUTHOR

David Kapell, dkapell@invisiblehand.net

=head1 SEE ALSO

perl(1).

=cut
